"""
    Small example program based upon the mastermind guessing game which was popular in the 70s (for those that remember that!)

    The program below continually plays games (note the infinite outer loop ('while True')) until '-1' is entered at any prompt.

    At the start of each game, the program chooses a random number between 1 - 100 (inclusive); you have to guess this number
    to win the game.

    After the mystery number has been chosen, you are asked to enter your guess. If you guess incorrectly, you are given
    a 'hint'; either to 'go Higher' or 'go Lower'.

    When you correctly guess the mystery number the program outputs the total number of guesses you took (for this game).

    ******************************************************************************
    Notes on how the code below might be modified/expanded further are at the end.
"""

import random

yourGuess = 0

guessesMade = 0

while yourGuess != -1:

    # \n means 'new line', e.g., output a blank line before this text.
    #
    print("\nNew Game starting. Guess a number in between 1 - 100 or -1 to exit")

    mysteryNumber = random.randint(1, 100)

    while(True):

        yourGuess = int(input("Enter a guess (or -1 to exit): "))

        if yourGuess == -1: break

        guessesMade += 1

        # Normally 'print' outputs a newline after it has output your text (the next print() will output on the
        # next line down). Using end = '' prevents this.
        #
        print(f'You guessed {yourGuess}', end = '')

        if yourGuess == mysteryNumber:

            print(f', that\'s  Correct!, you took {guessesMade} guesses to find it!')

            break

        # Will output something like 'You guessed 25 - go Lower'.
        #
        print(f' - go {"Higher" if yourGuess < mysteryNumber else "Lower"}')

print('Thank you for playing!')

"""
    Modifications:

    Any given Mystery number can be guessed in an optimal number of guesses, e.g., Say you have a range that's 1 - 1000,
    and that the mystery number is 1.  Your optimal guesses should proceed like this:

        500, 250, 125, 63, 32, 16, 8, 4, 2, 1

    1 is the worst possible case here and the number of guesses needed was 10. Note that at each step we throw away half of the
    remaining possible numbers.

    You can find the optimal number of guesses by taking the number of possible numbers - in our program that's 100 at the
    moment - and then by computing the log base 2 of that number, which is 6.64 guesses. You can't take fractional guesses, so
    we round that up using math.ceil(math.log2(maxNumber)) which gives us the maximum number of guesses required for 1 - 100,
    which is 7.

    Modify the code so that the upper number (the range 1 - ?????) is given by setting a variable, e.g., maxNumber = 100.
    Modify the prompts etc to reflect this, e.g., print(f'Guess a number in between 1 - {maxNumber} or -1 to exit').

    Modify the code so as to announce the optimal number of guesses required for the range in maxNumber. You'll need
    to 'import math' to compute the 'max number of guesses allowed' number using log2() etc. Then further modify the code
    so as to give the player a countdown of guesses remaining. Optionally, if they run out of guesses, reveal the mystery
    number and their 'guess count' and start again with a new mystery number.

    Modify the code so that the upper number in the 1 - ???? range is randomly generated for each game instead of being
    set at 100.
"""